package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.diagbot.dto.ConceptDetailDTO;
import com.diagbot.dto.ConceptIntroduceDTO;
import com.diagbot.entity.Concept;
import com.diagbot.entity.ConceptDetail;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.enums.PositionTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.impl.ConceptDetailServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.EntityUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.ParamConvertUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.ConceptBaseVO;
import com.diagbot.vo.ConceptIntroduceVO;
import com.diagbot.vo.ExistListByConceptIdsVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @Description:提示信息
 * @Author:zhaops
 * @time: 2019/5/6 15:03
 */
@Component
public class ConceptDetailFacade extends ConceptDetailServiceImpl {
    @Autowired
    private ConceptFacade conceptFacade;

    /**
     * 获取提示信息
     *
     * @param conceptIntroduceVO
     * @return
     */
    public ConceptIntroduceDTO getConceptDetail(ConceptIntroduceVO conceptIntroduceVO) {
        if (StringUtil.isBlank(conceptIntroduceVO.getName())) {
            throw new CommonException(CommonErrorCode.PARAM_IS_NULL, "请输入概念名称");
        }
        conceptIntroduceVO.setLibType(ParamConvertUtil.conceptConvert2Lib(conceptIntroduceVO.getType()));
        //验证存在并且获取概念基本信息
        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        BeanUtil.copyProperties(conceptIntroduceVO, conceptBaseVO);
        Concept concept = conceptFacade.exist(conceptBaseVO);
        //提示信息明细
        QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
        conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId())
                .orderByAsc("order_no");
        if (conceptIntroduceVO.getPosition() == null) {
            if (conceptIntroduceVO.getLibType().equals(LexiconTypeEnum.DRUGS.getKey())) {
                conceptIntroduceVO.setPosition(PositionTypeEnum.T5.getKey());
            } else if (conceptIntroduceVO.getLibType().equals(LexiconTypeEnum.SIDE_EFFECTS.getKey())) {
                conceptIntroduceVO.setPosition(PositionTypeEnum.T6.getKey());
            } else {
                conceptIntroduceVO.setPosition(PositionTypeEnum.T2.getKey());
            }
        }
        //position=0，返回所有位置的内容
        if (!conceptIntroduceVO.getPosition().equals(0)) {
            conceptDetailQueryWrapper.apply("find_in_set({0},position)", conceptIntroduceVO.getPosition());
        }
        if (ListUtil.isNotEmpty(conceptIntroduceVO.getTitles())) {
            conceptDetailQueryWrapper.in("title", conceptIntroduceVO.getTitles());
        }
        List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
        List<ConceptDetailDTO> conceptDetailDTOList = ListUtil.newArrayList();
        if (ListUtil.isNotEmpty(conceptDetailList)) {
            conceptDetailDTOList = BeanUtil.listCopyTo(conceptDetailList, ConceptDetailDTO.class);
        }
        ConceptIntroduceDTO conceptIntroduceDTO = new ConceptIntroduceDTO();
        conceptIntroduceDTO.setConceptId(concept.getId());
        conceptIntroduceDTO.setName(concept.getLibName());
        conceptIntroduceDTO.setDetails(conceptDetailDTOList);
        return conceptIntroduceDTO;
    }

    /**
     * 是否存在提示信息
     *
     * @param conceptIntroduceVO
     * @return
     */
    public Boolean exist(ConceptIntroduceVO conceptIntroduceVO) {
        conceptIntroduceVO.setLibType(ParamConvertUtil.conceptConvert2Lib(conceptIntroduceVO.getType()));
        //验证存在并且获取概念基本信息
        ConceptBaseVO conceptBaseVO = new ConceptBaseVO();
        BeanUtil.copyProperties(conceptIntroduceVO, conceptBaseVO);
        Concept concept = conceptFacade.exist(conceptBaseVO);
        QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
        conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .eq("concept_id", concept.getId());
        Integer res = this.count(conceptDetailQueryWrapper);
        if (res > 0) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * 是否存在提示信息-列表
     *
     * @param conceptList
     * @return
     */
    public Map<String, Map<Long, List<ConceptDetail>>> hasConDetail(List<Concept> conceptList) {
        Map<String, Map<Long, List<ConceptDetail>>> retMap = new LinkedHashMap<>();
        Map<Long, List<ConceptDetail>> retSubMap = new LinkedHashMap<>();
        if (ListUtil.isNotEmpty(conceptList)) {
            List<Long> conceptIds = conceptList.stream().map(concept -> concept.getId()).collect(Collectors.toList());
            QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
            conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey()).in("concept_id", conceptIds);
            List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
            if (ListUtil.isNotEmpty(conceptDetailList)) {
                Map<Long, List<ConceptDetail>> conceptDetailMap = EntityUtil.makeEntityListMap(conceptDetailList, "conceptId");
                for (Concept concept : conceptList) {
                    if (retMap.containsKey(concept.getLibName())) {
                        retSubMap = retMap.get(concept.getLibName());
                    }
                    if (retSubMap == null) {
                        retSubMap = new LinkedHashMap<>();
                    }
                    if (conceptDetailMap.containsKey(concept.getId()) && ListUtil.isNotEmpty(conceptDetailMap.get(concept.getId()))) {
                        retSubMap.put(concept.getLibType(), conceptDetailMap.get(concept.getId()));
                        retMap.put(concept.getLibName(), retSubMap);
                    }
                }
            }
        }
        return retMap;
    }

    /**
     * 根据概念id验证提示信息是否存在
     *
     * @param existListByConceptIdsVO
     * @return
     */
    public List<Long> existListByConceptIds(ExistListByConceptIdsVO existListByConceptIdsVO) {
        QueryWrapper<ConceptDetail> conceptDetailQueryWrapper = new QueryWrapper<>();
        conceptDetailQueryWrapper.eq("is_deleted", IsDeleteEnum.N.getKey())
                .in("concept_id", existListByConceptIdsVO.getConceptIds());
        List<ConceptDetail> conceptDetailList = this.list(conceptDetailQueryWrapper);
        if (ListUtil.isNotEmpty(conceptDetailList)) {
            List<Long> retConceptIds = conceptDetailList.stream().map(conceptDetail -> conceptDetail.getConceptId()).distinct().collect(Collectors.toList());
            return retConceptIds;
        } else {
            return null;
        }
    }
}