package com.diagbot.facade;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.diagbot.client.NeoServiceClient;
import com.diagbot.dto.ConceptBaseDTO;
import com.diagbot.dto.DiagnoseDetailSaveDTO;
import com.diagbot.dto.DiagnosePageDTO;
import com.diagbot.dto.LibraryForDiagnoseDTO;
import com.diagbot.dto.RespDTO;
import com.diagbot.dto.VerifyDataDTO;
import com.diagbot.entity.CommonParam;
import com.diagbot.entity.Diagnose;
import com.diagbot.entity.DiagnoseDetail;
import com.diagbot.entity.DiagnoseDetailSave;
import com.diagbot.entity.DiagnoseQuestion;
import com.diagbot.entity.wrapper.DiagnoseWrapper;
import com.diagbot.enums.DiagnoseFieldEnum;
import com.diagbot.enums.DiagnoseTypeEnum;
import com.diagbot.enums.HasQuestionEnum;
import com.diagbot.enums.InsertOrUpdateEnum;
import com.diagbot.enums.IsDeleteEnum;
import com.diagbot.enums.LexiconTypeEnum;
import com.diagbot.exception.CommonErrorCode;
import com.diagbot.exception.CommonException;
import com.diagbot.service.DiagnoseDetailService;
import com.diagbot.service.DiagnoseQuestionService;
import com.diagbot.service.impl.DiagnoseServiceImpl;
import com.diagbot.util.BeanUtil;
import com.diagbot.util.DateUtil;
import com.diagbot.util.DiagnoseTypeConvertUtil;
import com.diagbot.util.ListUtil;
import com.diagbot.util.RespDTOUtil;
import com.diagbot.util.StringUtil;
import com.diagbot.vo.DeleteDiagnoseVO;
import com.diagbot.vo.DiagnosePageVO;
import com.diagbot.vo.DiseaseIndexVO;
import com.diagbot.vo.GetDiagnoseDetailVO;
import com.diagbot.vo.NeoParamVO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Component
public class DiagnoseFacade extends DiagnoseServiceImpl {

    @Autowired
    ConceptFacade conceptFacade;
    @Autowired
    @Qualifier("diagnoseDetailServiceImpl")
    DiagnoseDetailService diagnoseDetailService;
    @Autowired
    @Qualifier("diagnoseQuestionServiceImpl")
    DiagnoseQuestionService diagnoseQuestionService;
    @Autowired
    DiagnoseDetailFacade diagnoseDetailFacade;
    @Autowired
    LibraryInfoFacade libraryInfoFacade;
    @Autowired
    NeoServiceClient neoServiceClient;

    /**
     * 诊断依据分页
     *
     * @param diagnosePageVO
     * @return
     */
    public IPage<DiagnosePageDTO> getListFac(DiagnosePageVO diagnosePageVO) {
        IPage<DiagnosePageDTO> res = this.getList(diagnosePageVO);
        if (res.getTotal() <= 0) {
            return res;
        }
        return res;
    }


    /**
     * 诊断依据删除
     *
     * @param deleteDiagnoseVO
     * @return
     */
    public void delete(DeleteDiagnoseVO deleteDiagnoseVO) {
        if (!(IsDeleteEnum.N.getKey().equals(deleteDiagnoseVO.getIsDeleted())
                || IsDeleteEnum.Y.getKey().equals(deleteDiagnoseVO.getIsDeleted()))) {
            throw new CommonException(CommonErrorCode.STATUS_IS_ERROR, "状态值错误【Y】或【N】");
        }
        Date now = DateUtil.now();
        this.update(new Diagnose(), new UpdateWrapper<Diagnose>()
                .eq("id", deleteDiagnoseVO.getId())
                .set("gmt_modified", now)
                .set("modifier", deleteDiagnoseVO.getModifier())
                .set("is_deleted", deleteDiagnoseVO.getIsDeleted())
                .set("neo_update", now)
                .set("remark", deleteDiagnoseVO.getRemark()));

        // 删除图谱数据
        NeoParamVO neoParamVO = new NeoParamVO();
        neoParamVO.setId(deleteDiagnoseVO.getId());
        neoServiceClient.deleteNeo(neoParamVO);
        RespDTO<Boolean> respDTO = neoServiceClient.updateNeo(neoParamVO);
        RespDTOUtil.respNGDeal(respDTO, "删除图谱数据失败【远程调用】");
    }


    /**
     * 诊断检索，过滤已添加的诊断
     *
     * @param diseaseIndexVO
     * @return
     */
    public List<ConceptBaseDTO> indexFac(DiseaseIndexVO diseaseIndexVO) {
        // 设置量表类型
        diseaseIndexVO.setLibType(LexiconTypeEnum.DIAGNOSIS.getKey());
        return this.index(diseaseIndexVO);
    }


    /**
     * 诊断依据获取详情
     *
     * @param getDiagnoseDetailVO
     * @return
     */
    public DiagnoseDetailSaveDTO getDetail(GetDiagnoseDetailVO getDiagnoseDetailVO) {
        DiagnoseDetailSaveDTO res = new DiagnoseDetailSaveDTO();
        List<DiagnoseDetail> detailList = diagnoseDetailFacade.list(new QueryWrapper<DiagnoseDetail>()
                .eq("diagnose_id", getDiagnoseDetailVO.getId())
                .eq("is_deleted", IsDeleteEnum.N.getKey())
                .orderByAsc("order_no")
        );
        for (DiagnoseDetail detail : detailList) {
            if (DiagnoseTypeEnum.Confirmed.getKey() == detail.getType() ||
                    DiagnoseTypeEnum.Examination.getKey() == detail.getType() ||
                    DiagnoseTypeEnum.Vigilant.getKey() == detail.getType()
            ) {
                res.getDisformula().add(detail);
            } else {
                res.getDisFeature().add(detail);
            }
        }
        return res;
    }


    /**
     * 诊断依据校验数据，新增或更新主表信息，覆盖明细表和问题词表
     *
     * @param diagnoseWrapper
     */
    public VerifyDataDTO verifyData(DiagnoseWrapper diagnoseWrapper) {
        VerifyDataDTO verifyDataDTO = new VerifyDataDTO();
        CommonParam param = initCommonParam();
        param.setPerson(diagnoseWrapper.getModifier());

        Diagnose diagnose = new Diagnose();
        BeanUtil.copyProperties(diagnoseWrapper, diagnose);
        if (diagnoseWrapper.getId() == null) { //新增
            diagnose.setCreator(param.getPerson());
            diagnose.setGmtCreate(param.getNow());
        } else { // 修改
            // 明细删除
            diagnoseDetailService.remove(new QueryWrapper<DiagnoseDetail>()
                    .eq("diagnose_id", diagnoseWrapper.getId())
            );
            // 删除问题词
            diagnoseQuestionService.remove(new QueryWrapper<DiagnoseQuestion>()
                    .eq("diagnose_id", diagnoseWrapper.getId())
            );
           param.setInsertOrUpdate(InsertOrUpdateEnum.Update.getKey());
        }
        diagnose.setGmtModified(param.getNow());
        this.saveOrUpdate(diagnose);


        // 导入词库数据
        List<LibraryForDiagnoseDTO> libraryForDiagnoseDTOS = libraryInfoFacade.getLibraryForDiagnose();

        // 新增明细
        List<DiagnoseDetail> diagnoseDetailList = new ArrayList<>();
        // 新增问题词
        List<DiagnoseQuestion> diagnoseQuestionList = new ArrayList<>();

        int i = 0, j = 0;
        // 遍历诊断特征
        for (DiagnoseDetailSave bean : diagnoseWrapper.getDisFeature()) {
            DiagnoseDetail diagnoseDetail = new DiagnoseDetail();
            BeanUtil.copyProperties(bean, diagnoseDetail);
            diagnoseDetail.setDiagnoseId(diagnose.getId());
            diagnoseDetail.setCreator(param.getPerson());
            diagnoseDetail.setGmtCreate(param.getNow());
            diagnoseDetail.setModifier(param.getPerson());
            diagnoseDetail.setGmtModified(param.getNow());
            diagnoseDetail.setDisName(diagnose.getDisName());
            diagnoseDetail.setOrderNo(++i);
            boolean flag = true;
            List<Integer> typeList = new ArrayList<>();

            // 校验编码
            // TODO

            // 校验标准词
            if (StringUtil.isNotEmpty(bean.getStandard())) {
                typeList = DiagnoseTypeConvertUtil.diagnoseType2LibType(bean.getType(), DiagnoseFieldEnum.standard.getKey());
                flag = verifyExist(libraryForDiagnoseDTOS, bean.getStandard(), typeList);
                if (!flag) {
                    diagnoseDetail.setVerifyStandard("1");
                    DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(diagnose.getId(), diagnose.getDisName(), bean.getType(),
                            bean.getStandard(), DiagnoseFieldEnum.standard.getKey(), initCommonParam(), ++j);
                    diagnoseQuestionList.add(diagnoseQuestion);
                }
            }

            // 校验关联词
            if (StringUtil.isNotEmpty(bean.getRelation())) {
                String[] arr = bean.getRelation().split("、");
                for (String s : arr) {
                    typeList = DiagnoseTypeConvertUtil.diagnoseType2LibType(bean.getType(), DiagnoseFieldEnum.relation.getKey());
                    flag = verifyExist(libraryForDiagnoseDTOS, s.trim(), typeList);
                    if (!flag) {
                        diagnoseDetail.setVerifyRelation("1");
                        DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(diagnose.getId(), diagnose.getDisName(), bean.getType(),
                                s.trim(), DiagnoseFieldEnum.relation.getKey(), initCommonParam(), ++j);
                        diagnoseQuestionList.add(diagnoseQuestion);
                    }
                }
            }

            // 校验结果
            if (StringUtil.isNotEmpty(bean.getResult())) {
                typeList = DiagnoseTypeConvertUtil.diagnoseType2LibType(bean.getType(), DiagnoseFieldEnum.result.getKey());
                flag = verifyExist(libraryForDiagnoseDTOS, bean.getResult(), typeList);
                if (!flag) {
                    diagnoseDetail.setVerifyResult("1");
                    DiagnoseQuestion diagnoseQuestion = createDiagnoseQuestion(diagnose.getId(), diagnose.getDisName(), bean.getType(),
                            bean.getStandard(), DiagnoseFieldEnum.standard.getKey(), initCommonParam(), ++j);
                    diagnoseQuestionList.add(diagnoseQuestion);
                }
            }
            diagnoseDetailList.add(diagnoseDetail);
        }

        // 遍历诊断公式
        for (DiagnoseDetailSave bean : diagnoseWrapper.getDisformula()) {
            DiagnoseDetail diagnoseDetail = new DiagnoseDetail();
            BeanUtil.copyProperties(bean, diagnoseDetail);
            diagnoseDetail.setDiagnoseId(diagnose.getId());
            diagnoseDetail.setCreator(param.getPerson());
            diagnoseDetail.setGmtCreate(param.getNow());
            diagnoseDetail.setModifier(param.getPerson());
            diagnoseDetail.setGmtModified(param.getNow());
            diagnoseDetail.setDisName(diagnose.getDisName());
            diagnoseDetail.setOrderNo(++i);
            // 校验公式
            // TODO
            diagnoseDetailList.add(diagnoseDetail);
        }
        diagnoseDetailService.saveBatch(diagnoseDetailList);

        // 有问题词逻辑
        if (ListUtil.isNotEmpty(diagnoseQuestionList)) {
            // 新增问题词数据
            diagnoseQuestionService.saveBatch(diagnoseQuestionList);
        }
        // 更新主表状态
        this.update(new UpdateWrapper<Diagnose>()
                .set("has_question", ListUtil.isNotEmpty(diagnoseQuestionList) ? HasQuestionEnum.HAS_QUESTION.getKey() : HasQuestionEnum.NO_QUESTION.getKey())
                .eq("id", diagnose.getId())
        );
        verifyDataDTO.setHasQuestion(ListUtil.isNotEmpty(diagnoseQuestionList) ? HasQuestionEnum.HAS_QUESTION.getKey() : HasQuestionEnum.NO_QUESTION.getKey());
        verifyDataDTO.setDiagnoseDetailList(diagnoseDetailList);
        return verifyDataDTO;
    }


    /**
     * 更新图谱(先校验，再更新)
     *
     * @param diagnoseWrapper
     * @return
     */
    public void  updateNeo(DiagnoseWrapper diagnoseWrapper) {
        VerifyDataDTO verifyDataDTO = verifyData(diagnoseWrapper);
        if (HasQuestionEnum.HAS_QUESTION.getKey() == verifyDataDTO.getHasQuestion()) {
            throw new CommonException(CommonErrorCode.SERVER_IS_ERROR, "有问题词存在，不能更新图谱");
        }
        // 更新图谱对接接口
        NeoParamVO neoParamVO = new NeoParamVO();
        neoParamVO.setId(diagnoseWrapper.getId());
        RespDTO<Boolean> respDTO = neoServiceClient.updateNeo(neoParamVO);
        RespDTOUtil.respNGDeal(respDTO, "更新图谱数据失败【远程调用】");
    }


    /**
     * 初始化参数
     *
     * @return
     */
    public CommonParam initCommonParam() {
        CommonParam param = new CommonParam();
        param.setNow(DateUtil.now());
        param.setInsertOrUpdate(InsertOrUpdateEnum.Insert.getKey());
        return param;
    }


    /**
     *  校验词库中是否存在
     *
     * @param library 词库数据
     * @param name 界面输入
     * @param type 转换类型
     * @return 校验结果
     */
    public boolean verifyExist(List<LibraryForDiagnoseDTO> library, String name, List<Integer> type) {
        for(Integer integer : type) {
            LibraryForDiagnoseDTO libraryForDiagnoseDTO = new LibraryForDiagnoseDTO();
            libraryForDiagnoseDTO.setName(name);
            libraryForDiagnoseDTO.setTypeId(integer);
            if (library.contains(libraryForDiagnoseDTO)) {
                return true;
            }
        }
        return false;
    }


    /**
     * 生成问题词
     *
     * @param disName
     * @param type
     * @param questionName
     * @param commonParam
     * @return
     */
    public DiagnoseQuestion createDiagnoseQuestion(Long diagnoseId, String disName, Integer type, String questionName, Integer questionType, CommonParam commonParam, Integer orderNo) {
        DiagnoseQuestion dq = new DiagnoseQuestion();
        dq.setCreator(commonParam.getPerson());
        dq.setGmtCreate(commonParam.getNow());
        dq.setModifier(commonParam.getPerson());
        dq.setGmtModified(commonParam.getNow());
        dq.setDisName(disName);
        dq.setDiagnoseId(diagnoseId);
        dq.setType(type);
        dq.setQuestionName(questionName);
        dq.setQuestionType(questionType);
        dq.setOrderNo(orderNo);
        return dq;
    }



}
